# Merijn Lamers-WUR|22-5-2023|Script 1/3
# Script to organize GC-MS-data

# Startup
rm(list = ls())
setwd("D:/Merijn Lamers/OneDrive - WageningenUR/WUR/Master/Thesis/Data/R/Final/")

# Function to check and install packages
check_install_packages <- function(pkg){
  new.pkg <- pkg[!(pkg %in% installed.packages()[,"Package"])]
  if (length(new.pkg)) {
    install.packages(new.pkg, dependencies = TRUE)
  }
  sapply(pkg, require, character.only = TRUE)
}

# Call the function to check and install packages
packages_used<-c("ggplot2", "dplyr", "FactoMineR", "factoextra","cowplot")
check_install_packages(packages_used)

# Check the versions of multiple packages
for (pkg in packages_used) {
  package_version <- packageVersion(pkg)
  print(paste(pkg, "version:", package_version))
}
# R: version.string R version 4.1.2 (2021-11-01)
#[1] "ggplot2 version: 3.4.2"
#[1] "dplyr version: 1.0.10"
#[1] "FactoMineR version: 2.7"
#[1] "factoextra version: 1.0.7"
#[1] "cowplot version: 1.1.1"

# Funtion to generate PCA plot, variables of the pca plot and eigenvalues scree plot 
generate_pca_plots <- function(data) {
  # Define colors for each location
  location_colors <- c("blue", "green", "red", "purple", "orange", "pink", "yellow", "brown", "black", "magenta", "cyan")
  
  # Get unique location names from the first 4 characters of the row names
  location_names <- unique(substr(rownames(data), 1, 4))
  
  # Assign a unique color to each location
  location_color_map <- setNames(location_colors[1:length(location_names)], location_names)
  colors <- unname(location_color_map[substr(rownames(data), 1, 4)])
  
  # Perform PCA and plot with colored samples
  pca <- PCA(data, scale.unit = TRUE, ncp = 5, graph = FALSE)
  
  # Convert PCA results to a data frame
  df_pca <- as.data.frame(pca$ind$coord[,1:2])
  
  # Add row names to the data frame
  df_pca$sample_names <- rownames(pca$ind$coord)
  
  # Add location names to the data frame
  df_pca$sample_locations <- substr(df_pca$sample_names, 1, 4)
  
  # Create the PCA plot using ggplot2
  pca_plot <- ggplot(df_pca, aes(x = Dim.1, y = Dim.2, color = sample_locations)) +
    geom_point(size = 4) +
    theme_bw()+
    scale_color_manual(values = location_color_map) +
    labs(title = "PCA plot with sample locations", x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))
  
  # Create a scree plot of the eigenvalues
  eigenvals_plot <- fviz_eig(pca, addlabels = TRUE)+
    theme_bw()
  eigenvals_plot
  # Plot the variables from the PCA and display only the labels from the top 10 contributing variables
  var_plot <- fviz_pca_var(pca, col.var = "contrib", gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"), repel = T, select.var = list(contrib = 8), )+
    theme_bw()
  
  # Arrange the plots using cowplot
  plot_grid(pca_plot, var_plot, eigenvals_plot)
}
# Function to normalize each col
normalize_col <- function(col) {
  col / max(col)
}

# Read in data
data_raw <- read.table("Data_14_4_filter.txt", header = TRUE)
data <- data_raw

# Apply normalization to data
#old normalized_data <- scale(data)
normalized_data <- (apply(data, 1, normalize_col))
data <- normalized_data

# Divide the data into subsets based on timepoint and grape
subset22 <- data[grep("^........22",rownames(data)),] #this is the only one used in the thesis
subset21 <- data[grep("^........21",rownames(data)),]
subset_1 <- data[grep("^......1", rownames(data)), ]
subset_2 <- data[grep("^......2", rownames(data)), ]
subset_3 <- data[grep("^......3", rownames(data)), ]
subset_4 <- data[grep("^......4", rownames(data)), ]

subsetnot1 <- rbind(subset_2,subset_3,subset_4)
subsetnot1_Sou <-subsetnot1[grep("SOU", rownames(subsetnot1)), ]

subset_Sou <- data[grep("SOU", rownames(data)), ]
subset_Ron <- data[grep("RON", rownames(data)), ]

subset_1S <- subset_Sou[grep("1", rownames(subset_Sou)), ]
subset_2S <- subset_Sou[grep("2", rownames(subset_Sou)), ]
subset_3S <- subset_Sou[grep("3", rownames(subset_Sou)), ]
subset_4S <- subset_Sou[grep("4", rownames(subset_Sou)), ]

# Select data and use it from here in the other scripts or create the PCA
data <- subset22
generate_pca_plots(data)



# Extract the location Flevoland or not flevoland
flevolocation <- substr(rownames(data),11,15)

# Extract timepoints from sample names
timepoints <- substr(rownames(data), 7, 7)

# Extract grape variety from sample name
grape <- substr(rownames(data), 4, 6)

# Extract the year of harvest
year <- substr(rownames(data),9,10)

# Perform PCA analysis and save into data-frame
pca <- PCA(data, scale.unit = TRUE, graph = FALSE)
df_pca <- as.data.frame(pca$ind$coord[,1:2])

# Plot the 8 most contributing factors to PC1 and PC2
fviz_eig(pca)
fviz_pca_var(pca, col.var = "contrib", gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"), repel = T, select.var = list(contrib = 8), )
test<-as.matrix((sort(pca$var$contrib[,1], decreasing = T)))

# Create plot and modify colours
ggplot(df_pca, aes(x = Dim.1, y = Dim.2, color = timepoints, shape = grape))+
  geom_point(size = 4) +
  #geom_point(aes(x = Dim.1, y = Dim.2, shape = flevolocation)) +
  scale_color_manual(values = c("green","yellow2", "orange", "red"))+
  #  stat_ellipse(level = 0.99)+
  theme_bw()+
  labs(x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))


# Plot same PCA but then coloured by grape variety 
ggplot(df_pca, aes(x = Dim.1, y = Dim.2, color = grape))+#, shape= timepoints))+
  geom_point(size = 4)+
  #  stat_ellipse(level = 0.8)+
  #  scale_shape_manual(values = c(15,19,17,18))+
  scale_color_manual(values = c("red3","chartreuse"))+
  labs(x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))
# labs(title = "PCA plot with sample grapes", x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))


# Plot the year
ggplot(df_pca, aes(x = Dim.1, y = Dim.2, color = grape))+
  geom_point(size = 5)+
  scale_color_manual(values = c("purple","green"))+
  theme_bw()+
  labs(x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))
# labs(title = "PCA plot with sample year", x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))

# Plot the location
ggplot(df_pca, aes(x = Dim.1, y = Dim.2, color = flevolocation))+
  geom_point(size = 5)+
  #  stat_ellipse()+
  #  guides(color="none")+
  theme_bw()+
  scale_color_manual(values = c("red","green"))+
  labs(title = "PCA plot with sample Flevoland", x = paste0("PC1 (", round(pca$eig[1,2], 1), "%)"), y = paste0("PC2 (", round(pca$eig[2,2], 1), "%)"))


# Combination plot timepoints, grape, felvoland or not
ggplot(df_pca, aes(x = Dim.1, y = Dim.2, color = timepoints, shape = grape)) +
  geom_point(size = 8) +
  scale_shape_manual(values = c(16,17))+
  geom_point(data = subset(df_pca, flevolocation == "Flevo"),
             aes(x = Dim.1, y = Dim.2),
             shape = 16, size = 4, color = "black") +
  geom_point(data = subset(df_pca, flevolocation == "nonfl"),
             aes(x = Dim.1, y = Dim.2),
             shape = 1, size = 5, color = "black") +
  geom_point(data = subset(df_pca, flevolocation == "nonfl"),
             aes(x = Dim.1, y = Dim.2),
             shape = 16, size = 4.2, color = "white") +
  scale_color_manual(values = c("green", "yellow2", "orange", "red")) +
  theme_bw() +
  labs(x = paste0("PC1 (", round(pca$eig[1, 2], 1), "%)"),
       y = paste0("PC2 (", round(pca$eig[2, 2], 1), "%)"))

# For violin plots of the GC-MS data

# install and load the vioplot package
# install.packages("vioplot") version used: 0.4.0
library(vioplot)

normalized_data <- normalized_matrix

# create a violin plot of all the rows dataset
par(mar = c(16, 5, 1, 1), mgp = c(4,1,0))
vioplot(t(data_raw[,]), las = 2, col = "darkred", ylab = "Peak area (counts/min)")
par(mar = c(16, 5, 1, 1), mgp = c(2.5,1,0))
vioplot(normalized_data,las =2, col = "darkblue", ylab = "Normalized peak area", cex.lab = 5)
