# Merijn Lamers|28-6-2023|2/3
# Script for OTUtable with taxonomy
# Startup, clear all data
rm(list = ls())
setwd("C:/Users/Lamer043/OneDrive - Wageningen University & Research/WUR/Master/Thesis/Paper/2024 draft/Data+Scripts/")

# Function to and load packages
check_and_load_packages <- function(packages) {
  for (pkg in packages) {
    if (!require(pkg, character.only = TRUE)) {
      install.packages(pkg)
      library(pkg, character.only = TRUE)
    } else {
      library(pkg, character.only = TRUE)
    }
  }
}

# Packages used, version in README file
packages <- c("phyloseq","phyloseq.extended","Biostrings", "vegan", "taxize", "ggplot2", "ggfortify", "factoextra", "psych", "gridExtra", "rstatix", "DESeq2", "DT", "corncob", "ggraph", "microbiome", "insect", "dplyr", "ggthemes", "pheatmap","ggrepel")
check_and_load_packages(packages)

# Check version with README
for (pkg in packages) {
  package_version <- packageVersion(pkg)
  print(paste(pkg, "version:", package_version))
}

# Read in all the data
OTU <-read.table("Only_otu_tax.txt", header=TRUE, sep="\t")
rownames(OTU) <- OTU[,1]
OTU <- OTU[,-1]
rownames(OTU)<- sub("centroid=([^;]+).*", "\\1",rownames(OTU))
otutable <- otu_table(as.matrix(OTU), taxa_are_rows=TRUE)
metadatatab <-read.table("metadata.txt", header=TRUE, sep="\t", row.names=1)
metadatatab <- sample_data(metadatatab)
taxedit <- read.csv2("tax.csv", header = T)
rownames(taxedit)<-taxedit[,1]
taxedit<-taxedit[,-1] 
rownames(taxedit)<- sub("centroid=([^;]+).*", "\\1",rownames(taxedit))
taxtable <- tax_table(as.matrix(taxedit))
voccor<-read.table(file = "vocmetadata.txt", header = T)

# Create Phyloseq object
Flevoyeast <- phyloseq(otutable, taxtable, metadatatab)

# Remove the samples that were stored incorrect and pure cultures
Flev_Filt <- subset_samples(Flevoyeast, Farm != "KLE" & Farm != "Cult")

# Data rarefaction
set.seed(143)
yeast_rare <- rarefy_even_depth(Flev_Filt, sample.size = 2000, rngseed = 143)

# Create subsets
corphy<-subset_samples(Flevoyeast, GC.MS == "yes" & Year=="Y2022")
yeast_timeclust <- merge_samples(yeast_rare, "Timepoint")
Y2021<-subset_samples(yeast_rare, Year=="Y2021" )

# Select data for analysis 
phy <- yeast_rare

# Alpha diversity plots
plot_richness(phy, color = "Farm", x = "Timepoint") +
  geom_boxplot()

yeast_richness <- estimate_richness(phy, measures = c("Observed","Chao1", "InvSimpson"))

plot_richness(phy, color = "Timepoint", x = "Timepoint", measures = c("Observed","Chao1", "InvSimpson")) +
  geom_violin(trim = F)+
  geom_boxplot(width=0.15, color="black")+
  geom_point()+
  theme(text = element_text(size=20))

# Using the Radfit function from Vegan to see the evenness per sample and of the population
pdf("evenness_plots.pdf", width = 25, height = 30)
plot(radfit(t(otu_table(phy)), ))
dev.off()
plot(radfit(sort(rowSums(otu_table(phy)),decreasing = T), log="y", xlab="Rank", ylab="Abundance"))

# Fast way to save the Richness and then find if there are significant factors
alpha.diversity <- estimate_richness(phy)
#write.table(alpha.diversity, file = "diversity.txt")
data <- cbind(sample_data(phy),alpha.diversity)
test.anova <- aov(InvSimpson~Timepoint, data)
summary(test.anova)

# Create a bar plot to look into the samples at family level can be other levels
phyloseq.extended::plot_composition(phy, taxaRank2 = "family", numberOfTaxa = 10, spread=T)+
  facet_grid(~Timepoint,scales = "free_x", space = "free")+
  theme(axis.text.x = element_text(size = 7,colour = "black", vjust = 0.4), 
      axis.text.y = element_text(size = 8))+
  ylab("Relative abundance")

# Bar plot of one specific tax level e.g. see what Saccharomyces strains there are in Saccharomyces
phyloseq.extended::plot_composition(phy, taxaRank1 = "family", 
                                    taxaSet1 = "Saccharomycetaceae", taxaRank2 = "OTU", fill = "OTU", numberOfTaxa = 10)+
  facet_grid(~Timepoint,scales = "free_x", space = "free")+
  scale_fill_stata()+
  theme(axis.text.x = element_text(vjust = 0.4))

# Create a Bray-Curtis dissimularity matrix and create a heatmap and dend with that
phy_bray <- phyloseq::distance(phy, method = "bray")
heatmap(as.matrix(phy_bray))
par(mfcol = c(1,3))
  plot(hclust(phy_bray, method = "complete"))
  plot(hclust(phy_bray, method = "ward.D2"))
  plot(hclust(phy_bray, method = "single"))
par(mfcol=c(1,1))

# Visualize the ordination with MDS or other prefered method (PCoA, NMDS)
ord <- ordinate(phy, method = "MDS", distance = phy_bray)
plot_ordination(phy,ord, color = "Timepoint", shape = "Grape") +
  scale_color_manual(values = c("green", "yellow2", "orange", "red"))+
  theme_bw()+
  #scale_size_manual(size=2)+
  #stat_ellipse(level = 0.8)+
  geom_point(size = 5) #+
#theme(text = element_text(size = 10))

# Plot for paper:
test <- cbind(ord$vectors[,1:2], sample_data(phy))
ggplot(test, aes(x=Axis.1, y=Axis.2, color= Timepoint, shape=Grape))+
  geom_point(size = 8) +
  scale_shape_manual(values = c(16,17))+
  geom_point(data = subset(test, Flevoland == "Yes"),
             aes(x = Axis.1, y = Axis.2),
             shape = 16, size = 4, color = "black") +
  geom_point(data = subset(test, Flevoland == "No"),
             aes(x = Axis.1, y = Axis.2),
             shape = 1, size = 5, color = "black") +
  geom_point(data = subset(test, Flevoland == "No"),
             aes(x = Axis.1, y = Axis.2),
             shape = 16, size = 4.2, color = "white") +
  scale_color_manual(values = c("green", "yellow2", "orange", "red"))+
  theme_bw()+
  labs(x = paste0("Dimension 1 (", round(100*ord$values[1,2], 1), "%)"),
       y = paste0("Dimension 2 (", round(100*ord$values[2,2], 1), "%)"))

# Script for linking environmental data to seq data
phy <- corphy
phy <- transform_sample_counts(phy, function(x) x/sum(x))

# Take sp. whith a abundance of a set mean percentage, extract otu table
phy_filt <- filter_taxa(phy,function(x) mean(x) > 0.01, TRUE)
otutable <- otu_table(phy_filt)
otutable<-as.data.frame(otutable)
otutable<-as.matrix(otutable)
otutable<-t(otutable)
sampledf <- data.frame(sample_data(phy_filt))

# Check if RDA or CCA is the method to be used
dca<- decorana(otutable)
dca #look at axis length of DCA1, <3 RDA >4 CCA

# Select envdata and normalise it and preform CCA/RDA
df <- data.frame(scale(sampledf[,c(9:36)])) 
ord_all <- cca(otutable ~ ., data=df)
ord_all <- rda(otutable ~.,  data=df )


# Plot the biplot type can be text, points, none. scaling= sites, species, symmetric or none
plot(ord_all, type="text", scaling = "symmetric")
plot(ord_all, type="text", scaling = "none")
plot(ord_all, type="text", scaling = "sites")
plot(ord_all, type="text", scaling = "species")

# Check the Variance Inflation Factors 
temp <- vif.cca(ord_all)
temp
select_para <- na.omit(names(temp[temp < 200])) # normally cutoff < 20
select_para

ord <- cca(otutable ~ ., df[,select_para]) 
plot(ord, type = "text", scaling = "symmetric")

# Extract the OTU table and metadata from the phyloseq object
otu_table <- otu_table(phy)
otu_table<- t(as.data.frame(otu_table))
col_sums <- colSums(otu_table)
plot(sort(col_sums, decreasing = T), log="y")
col_to_remove <- which(col_sums < 0.05)
otu_table <- otu_table[, -col_to_remove]
taxcor<-colnames(otu_table)
taxcor<-taxedit[rownames(taxedit) %in% taxcor,]
taxcor[is.na(taxcor)]<- "Other"
voc_metadata <- meta(phy)
voc_metadata <- voc_metadata[,9:36]

# Correlation analysis via Spearman
correlations <- matrix(NA, nrow = ncol(otu_table), ncol = ncol(voc_metadata))
colnames(correlations) <- colnames(voc_metadata)
rownames(correlations) <- colnames(otu_table)
p_values <- correlations
for (i in 1:ncol(otu_table)) {
  for (j in 1:ncol(voc_metadata)) {
    res <- cor.test(otu_table[, i], voc_metadata[, j], method = "spearman")
    correlations[i, j] <- res$estimate
    p_values[i, j] <- res$p.value
  }
}

# Convert the correlations and p-values matrices to data frames
correlation_df <- data.frame(correlations)
p_values_df <- data.frame(p_values)
correlation_df<-na.omit(correlation_df)

# Fix the names 
correct_names<-colnames(read.table("metadata.txt", header=TRUE, sep="\t", row.names=1, check.names = F)[9:36])
colnames(correlation_df)<-correct_names
rownames(voccor)<-correct_names

# Create a heatmap with colors that correspond to the OTUs and metadata 
t_cor_df<-t(correlation_df)
my_color = list(
  genus = c(Brettanomyces = "red", Hanseniaspora = "blue",Other = "black", Saccharomyces = "green"),
  family = c(Other = "black", Saccharomycodaceae = "yellow2", Pichiaceae = "purple", Saccharomycetaceae = "salmon" ),
  group = c(Acid = "lightgreen", Ester= "orange", Alcohol = "darkred", Keton = "cornflowerblue", Anhydride = "cyan", Aliphatic = "deeppink" )
)

pheatmap(t_cor_df, cutree_cols = 4, cutree_rows = 3,annotation_row = voccor, 
         annotation_col = taxcor[,5:6], annotation_colors = my_color, fontsize = 15)

# Script for Rarefaction curves
# Select data
data<-otu_table(Flev_Filt)
data<-as.data.frame(data)
data<-as.matrix(data)

# Create rarefaction curves with sample set of 100 per interval
rarecurve_data <- rarecurve(t(data), step = 100, sample = min(rowSums(data)), tidy = T)

# Find where they end so it will be possible to plot the names later
max_points <- rarecurve_data %>%
  group_by(Site) %>%
  slice_max(Sample)

# Plot the rarefaction curves
ggplot(data = rarecurve_data, aes(x = Sample, y = Species, group = Site, color = Site), cex=1.5) +
  geom_line(linewidth=1.5) +
  theme_minimal() +
  xlab("Sequencing effort (reads sampled)") +
  ylab("Observed OTUs") +
  ggtitle("Rarefaction curve") +
  scale_color_discrete(name = "Site") +
  theme(legend.position = "none") + # Remove the legend
  geom_text_repel(data = max_points, aes(x = Sample, y = Species, label = Site), force = 1) # Add labels to the end of each curve


ggplot(data = rarecurve_data, aes(x = Sample, y = Species, group = Site, color = Site), cex=1.5) +
  geom_line(linewidth=1.5) +
  theme_minimal() +
  xlab("Sequencing effort (reads sampled)") +
  ylab("Observed OTUs") +
  ggtitle("Rarefaction curve") +
  scale_color_discrete(name = "Site") +
  theme(legend.position = "none") + # Remove the legend
  geom_text_repel(data = max_points, aes(x = Sample, y = Species, label = Site), force = 1) + # Add labels to the end of each curve
  coord_cartesian(xlim = c(0, 5000)) # Set x-axis limits from 0 to 5000



